;(function() {
	if (window.WebViewJavascriptBridge||!document.documentElement) {
	    console.log("window.WebViewJavascriptBridge已存在")
	    return
	}
	// 改变src属性来通知原生的iframe
	var messagingIframe
	// 发送的message队列
	var sendMessageQueue = []
	// 接受消息队列：在init之前收到消息时候保存，init后会全部处理
	var receiveMessageQueue = []
	// 保存h5注册handler，格式 key：handlerName, value:handler
	var messageHandlers = {}

	// scheme相关
	var CUSTOM_PROTOCOL_SCHEME = 'wvjbscheme'
	var QUEUE_HAS_MESSAGE = '__WVJB_QUEUE_MESSAGE__'

	// 响应回调集合，key:callbackId, value:responseCallback
	var responseCallbacks = {}
	// 生成不同callbackId的拼接参数
	var uniqueId = 1

	var lastCallTime = 0;
    var stoId = null;
    var FETCH_QUEUE = 40;

	// 动态创建 iframe---messagingIframe
	function _createQueueReadyIframe(doc) {
		messagingIframe = doc.createElement('iframe')
		messagingIframe.style.display = 'none'
		doc.documentElement.appendChild(messagingIframe)
	}

    // 初始化jsbridge，messageHandler回调，作为 jsbridge可用标志
	function init(messageHandler) {
		if (WebViewJavascriptBridge._messageHandler) { return }
		WebViewJavascriptBridge._messageHandler = messageHandler
		// 如果在初始化之前收到了消息，则初始化后全部处理掉
		var receivedMessages = receiveMessageQueue
		receiveMessageQueue = null
		for (var i=0; i<receivedMessages.length; i++) {
			_dispatchMessageFromObjC(receivedMessages[i])
		}
	}

    // 发送数据， 最终的消息结构是：{ data:data, callbackId:cb_1_1232323131313 }, responseCallback被保存在responseCallbacks中
	function send(data, responseCallback) {
		_doSend({ data:data }, responseCallback)
	}

	// h5注册handler，保存到messageHandlers中
	function registerHandler(handlerName, handler) {
		messageHandlers[handlerName] = handler
	}

    // 判断h5注册的handler是否存在
	function containHandler(handlerName) {
	    return messageHandlers[handlerName] != null
	}

    // 注销handler
	function unregisterHandler(handlerName) {
	    return messageHandlers[handlerName] = null
	}

	// h5调用原生handler，消息结构：{ handlerName:handlerName, data:data, callbackId:cb_1_1232323131313 }, responseCallback被保存在responseCallbacks中
	function callHandler(handlerName, data, responseCallback) {
		_doSend({ handlerName:handlerName, data:data }, responseCallback)
	}
	
	function _doSend(message, responseCallback) {
		if (responseCallback) {
			// 如果有回调，则产生一个唯一的回调id： 如 cb_1_1232323131313
			var callbackId = 'cb_'+(uniqueId++)+'_'+new Date().getTime()
			// key为 callbackId, value为回调，保存到 responseCallbacks中
			responseCallbacks[callbackId] = responseCallback
			// 为消息添加回调callbackId， 消息数据如{ handlerName:handlerName, data:data , callbackId:cb_1_1232323131313}
			message['callbackId'] = callbackId
		}
		// 将发送的消息添加到发送队列  sendMessageQueue
		sendMessageQueue.push(message)

        // 解决h5触发次数太多导致的原生 回调未触发问题，通过截流方式处理
        // 详细参考：https://blog.csdn.net/weixin_34185512/article/details/91421901
		if (new Date().getTime() - lastCallTime < FETCH_QUEUE) {
            if (!stoId) {
                stoId = setTimeout(_callNativeWithIframeSrc, FETCH_QUEUE);
            }
            return;
        }

		_callNativeWithIframeSrc();
	}

	function _callNativeWithIframeSrc() {
	    lastCallTime = new Date().getTime();
        stoId = null;
	    // 通过 设置iframe 的src为 wvjbscheme://__WVJB_QUEUE_MESSAGE__,触发原生scheme
    	messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://' + QUEUE_HAS_MESSAGE
	}

    // 获取发送消息队列字符串
	function _fetchQueue() {
		var messageQueueString = JSON.stringify(sendMessageQueue)
		sendMessageQueue = []
		return messageQueueString
	}

    // h5处理native发送来的message字符串
	function _dispatchMessageFromObjC(messageJSON) {
		setTimeout(function _timeoutDispatchMessageFromObjC() {
			var message = JSON.parse(messageJSON)
			var messageHandler

			if (message.responseId) {
				// h5发送带回调的消息给native，native响应处理
				// 通过native发送的message的响应id获取 响应回调
				var responseCallback = responseCallbacks[message.responseId]
				if (!responseCallback) { return; }
				// 响应回调调用，参数为消息的回调数据
				responseCallback(message.responseData)
				// 响应回调处理完成后，responseCallbacks中去除已处理的回调
				delete responseCallbacks[message.responseId]
			} else {
				var responseCallback
				if (message.callbackId) {
					// native发送带回调的消息给h5
					var callbackResponseId = message.callbackId
					responseCallback = function(responseData) {
						_doSend({ responseId:callbackResponseId, responseData:responseData })
					}
				}

				// 默认为 init时的 _messageHandler作为 message的handler
				var handler = WebViewJavascriptBridge._messageHandler
				// 如果 原生callHandler，则用对应名字的handler作为消息处理handler
				if (message.handlerName) {
					handler = messageHandlers[message.handlerName]
				}
				
				try {
					// 处理handler
					handler(message.data, responseCallback)
				} catch(exception) {
					if (typeof console != 'undefined') {
						console.log("WebViewJavascriptBridge: WARNING: javascript handler threw.", message, exception)
					}
				}
			}
		})
	}

	// h5处理原生发来的消息
	function _handleMessageFromObjC(messageJSON) {
		if (receiveMessageQueue) {
			// 如果初始化之前，收到消息，则保存到接受消息队列中
			receiveMessageQueue.push(messageJSON)
		} else {
			// 如果初始化后，则直接处理消息
			_dispatchMessageFromObjC(messageJSON)
		}
	}

	window.WebViewJavascriptBridge = {
		init: init,
		send: send,
		registerHandler: registerHandler,
		callHandler: callHandler,
		containHandler: containHandler,
		_fetchQueue: _fetchQueue,
		_handleMessageFromObjC: _handleMessageFromObjC
	}

	var doc = document
	_createQueueReadyIframe(doc)
	// 发送h5 bridge 准备就绪事件回调
	var readyEvent = doc.createEvent('Events')
	readyEvent.initEvent('WebViewJavascriptBridgeReady')
	readyEvent.bridge = WebViewJavascriptBridge
	doc.dispatchEvent(readyEvent)
	console.log("WebViewJavascriptBridgeReady")
})();
